package nl.iobyte.themeparkconnector.api.operator.objects;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themeparkconnector.ThemeParkConnector;
import nl.iobyte.themeparkconnector.api.ThemeParkConnectorAPI;
import nl.iobyte.themepark.api.attraction.objects.Attraction;
import nl.iobyte.themeparkconnector.api.client.ClientConnection;
import nl.iobyte.themeparkconnector.api.event.operator.OperatorConnectEvent;
import nl.iobyte.themeparkconnector.api.event.operator.OperatorDisconnectEvent;
import nl.iobyte.themeparkconnector.api.operator.objects.panel.OperatorPanel;
import java.util.UUID;

public class AttractionOperator {

    private UUID client_uuid;
    private final String attraction_id, permission, start_command, stop_command;

    public AttractionOperator(Attraction attraction, String permission, String start_command, String stop_command) {
        this.attraction_id = attraction.getID();
        this.permission = permission;
        this.start_command = start_command;
        this.stop_command = stop_command;
    }

    public AttractionOperator(String attraction_id, String permission, String start_command, String stop_command) {
        this.attraction_id = attraction_id;
        this.permission = permission;
        this.start_command = start_command;
        this.stop_command = stop_command;
    }

    public UUID getClientUUID() {
        return client_uuid;
    }

    public ClientConnection getClientConnection() {
        return ThemeParkConnector.getInstance().getAPI().getNetworkingService().getClient(client_uuid);
    }

    public boolean canClientConnect(ClientConnection connection) {
        if(connection == null)
            return false;

        if(client_uuid != null)
            return false;

        if(hasPermission())
            return connection.getPlayer().hasPermission(permission);

        return true;
    }

    public void setClient(UUID client_uuid) {
        ThemeParkConnectorAPI api = ThemeParkConnector.getInstance().getAPI();
        if(client_uuid == null) {
            api.getEventDispatcher().call(new OperatorDisconnectEvent(
                    this,
                    api.getNetworkingService().getClient(this.client_uuid)
            ));
        } else {
            api.getEventDispatcher().call(new OperatorConnectEvent(
                    this,
                    api.getNetworkingService().getClient(client_uuid)
            ));
        }

        this.client_uuid = client_uuid;
    }

    public String getAttractionID() {
        return attraction_id;
    }

    public Attraction getAttraction() {
        return ThemePark.getInstance().getAPI().getAttractionService().getAttraction(attraction_id);
    }

    public String getPermission() {
        return permission;
    }

    public boolean hasPermission() {
        return permission != null && !permission.isEmpty();
    }

    public String getStartCommand() {
        return start_command;
    }

    public String getStopCommand() {
        return stop_command;
    }

    public boolean isAvailable() {
        return client_uuid == null;
    }

    public OperatorPanel getPanel() {
        return ThemeParkConnector.getInstance().getAPI().getOperatorService().getPanel(attraction_id);
    }

}
