package nl.iobyte.themeparkconnector.api.packet.adapter;

import com.google.gson.*;
import nl.iobyte.themeparkconnector.ThemeParkConnector;
import nl.iobyte.themeparkconnector.api.packet.objects.AbstractPacketPayload;
import nl.iobyte.themeparkconnector.logger.ThemeParkConnectorLogger;
import java.lang.reflect.Type;

public class PacketPayloadAdapter implements JsonSerializer<AbstractPacketPayload>, JsonDeserializer<AbstractPacketPayload> {

    /**
     * Serializes AbstractPacketPayload to json
     * @param src AbstractPacketPayload
     * @param typeOfSrc Type
     * @param context JsonSerializationContext
     * @return JsonElement
     */
    @Override
    public JsonElement serialize(AbstractPacketPayload src, Type typeOfSrc, JsonSerializationContext context) {
        JsonObject result = new JsonObject();
        result.add("type", new JsonPrimitive(src.getClass().getName()));
        result.add("payload", context.serialize(src, src.getClass()));

        return result;
    }

    /**
     * Deserializes AbstractPacketPayload to correct type
     * @param element JsonElement
     * @param typeOf Type
     * @param context JsonDeserializationContext
     * @return AbstractPacketPayload
     * @throws JsonParseException Exception that might occur
     */
    @Override
    public AbstractPacketPayload deserialize(JsonElement element, Type typeOf, JsonDeserializationContext context) throws JsonParseException {
        try {
            JsonObject object = element.getAsJsonObject();
            String type = object.get("type").toString().replace("\"", "");
            JsonElement payload = object.get("payload");

            if(!type.contains("."))
                type = "nl.iobyte.themeparkconnector.api.packet.payloads."+type;

            return context.deserialize(payload, Class.forName(type));
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

}
